@@ -30,42 +30,29 @@
 
 #include <internal.h>
 
-#define ATTRD_PROTOCOL_VERSION "1"
+/*
+ * Legacy attrd (all pre-1.1.11 Pacemaker versions, plus all versions when using
+ * heartbeat, CMAN, or corosync-plugin stacks) is unversioned.
+ *
+ * With atomic attrd, each attrd will send ATTRD_PROTOCOL_VERSION with every
+ * peer request and reply. Currently, there is no way to know the minimum
+ * version supported by all peers, which limits its usefulness.
+ *
+ * Protocol  Pacemaker  Significant changes
+ * --------  ---------  -------------------
+ *     1       1.1.11   ATTRD_OP_UPDATE (F_ATTRD_ATTRIBUTE only),
+ *                      ATTRD_OP_PEER_REMOVE, ATTRD_OP_REFRESH, ATTRD_OP_FLUSH,
+ *                      ATTRD_OP_SYNC, ATTRD_OP_SYNC_RESPONSE
+ *     1       1.1.13   ATTRD_OP_UPDATE (with F_ATTR_REGEX), ATTRD_OP_QUERY
+ *     1       1.1.15   ATTRD_OP_UPDATE_BOTH, ATTRD_OP_UPDATE_DELAY
+ *     2       1.1.17   ATTRD_OP_CLEAR_FAILCOUNT
+ */
+#define ATTRD_PROTOCOL_VERSION "2"
 
 int last_cib_op_done = 0;
 char *peer_writer = NULL;
 GHashTable *attributes = NULL;
 
-typedef struct attribute_s {
-    char *uuid; /* TODO: Remove if at all possible */
-    char *id;
-    char *set;
-
-    GHashTable *values;
-
-    int update;
-    int timeout_ms;
-
-    /* TODO: refactor these three as a bitmask */
-    bool changed; /* whether attribute value has changed since last write */
-    bool unknown_peer_uuids; /* whether we know we're missing a peer uuid */
-    gboolean is_private; /* whether to keep this attribute out of the CIB */
-
-    mainloop_timer_t *timer;
-
-    char *user;
-
-} attribute_t;
-
-typedef struct attribute_value_s {
-        uint32_t nodeid;
-        gboolean is_remote;
-        char *nodename;
-        char *current;
-        char *requested;
-} attribute_value_t;
-
-
 void write_attribute(attribute_t *a);
 void write_or_elect_attribute(attribute_t *a);
 void attrd_peer_update(crm_node_t *peer, xmlNode *xml, const char *host, bool filter);
@@ -213,9 +200,6 @@ void
 attrd_client_update(xmlNode *xml)
 {
     attribute_t *a = NULL;
-    attribute_value_t *v = NULL;
-    char *key = crm_element_value_copy(xml, F_ATTRD_KEY);
-    char *set = crm_element_value_copy(xml, F_ATTRD_SET);
     char *host = crm_element_value_copy(xml, F_ATTRD_HOST);
     const char *attr = crm_element_value(xml, F_ATTRD_ATTRIBUTE);
     const char *value = crm_element_value(xml, F_ATTRD_VALUE);
@@ -227,7 +211,7 @@ attrd_client_update(xmlNode *xml)
         regex_t *r_patt = calloc(1, sizeof(regex_t));
 
         crm_debug("Setting %s to %s", regex, value);
-        if (regcomp(r_patt, regex, REG_EXTENDED)) {
+        if (regcomp(r_patt, regex, REG_EXTENDED|REG_NOSUB)) {
             crm_err("Bad regex '%s' for update", regex);
 
         } else {
@@ -243,8 +227,6 @@ attrd_client_update(xmlNode *xml)
             }
         }
 
-        free(key);
-        free(set);
         free(host);
         regfree(r_patt);
         free(r_patt);
@@ -252,8 +234,6 @@ attrd_client_update(xmlNode *xml)
 
     } else if (attr == NULL) {
         crm_err("Update request did not specify attribute or regular expression");
-        free(key);
-        free(set);
         free(host);
         return;
     }
@@ -269,30 +249,14 @@ attrd_client_update(xmlNode *xml)
 
     /* If value was specified using ++ or += notation, expand to real value */
     if (value) {
-        int offset = 1;
-        int int_value = 0;
-        static const int plus_plus_len = 5;
-
-        if ((strlen(value) >= (plus_plus_len + 2)) && (value[plus_plus_len] == '+')
-            && ((value[plus_plus_len + 1] == '+') || (value[plus_plus_len + 1] == '='))) {
+        if (attrd_value_needs_expansion(value)) {
+            int int_value;
+            attribute_value_t *v = NULL;
 
             if (a) {
                 v = g_hash_table_lookup(a->values, host);
             }
-            if (v) {
-                int_value = char2score(v->current);
-            }
-
-            if (value[plus_plus_len + 1] != '+') {
-                const char *offset_s = value + (plus_plus_len + 2);
-
-                offset = char2score(offset_s);
-            }
-            int_value += offset;
-
-            if (int_value > INFINITY) {
-                int_value = INFINITY;
-            }
+            int_value = attrd_expand_value(value, (v? v->current : NULL));
 
             crm_info("Expanded %s=%s to %d", attr, value, int_value);
             crm_xml_add_int(xml, F_ATTRD_VALUE, int_value);
@@ -310,13 +274,74 @@ attrd_client_update(xmlNode *xml)
     crm_debug("Broadcasting %s[%s] = %s%s", attr, host, value,
               ((election_state(writer) == election_won)? " (writer)" : ""));
 
-    free(key);
-    free(set);
     free(host);
 
     send_attrd_message(NULL, xml); /* ends up at attrd_peer_message() */
 }
 
+/*!
+ * \internal
+ * \brief Respond to client clear-failure request
+ *
+ * \param[in] xml         Request XML
+ */
+void
+attrd_client_clear_failure(xmlNode *xml)
+{
+#if 0
+    /* @TODO This would be most efficient, but there is currently no way to
+     * verify that all peers support the op. If that ever changes, we could
+     * enable this code.
+     */
+    if (all_peers_support_clear_failure) {
+        /* Propagate to all peers (including ourselves).
+         * This ends up at attrd_peer_message().
+         */
+        send_attrd_message(NULL, xml);
+        return;
+    }
+#endif
+
+    const char *rsc = crm_element_value(xml, F_ATTRD_RESOURCE);
+    const char *op = crm_element_value(xml, F_ATTRD_OPERATION);
+    const char *interval_s = crm_element_value(xml, F_ATTRD_INTERVAL);
+
+    /* Map this to an update */
+    crm_xml_add(xml, F_ATTRD_TASK, ATTRD_OP_UPDATE);
+
+    /* Add regular expression matching desired attributes */
+
+    if (rsc) {
+        char *pattern;
+
+        if (op == NULL) {
+            pattern = crm_strdup_printf(ATTRD_RE_CLEAR_ONE, rsc);
+
+        } else {
+            int interval = crm_get_interval(interval_s);
+
+            pattern = crm_strdup_printf(ATTRD_RE_CLEAR_OP,
+                                        rsc, op, interval);
+        }
+
+        crm_xml_add(xml, F_ATTRD_REGEX, pattern);
+        free(pattern);
+
+    } else {
+        crm_xml_add(xml, F_ATTRD_REGEX, ATTRD_RE_CLEAR_ALL);
+    }
+
+    /* Make sure attribute and value are not set, so we delete via regex */
+    if (crm_element_value(xml, F_ATTRD_ATTRIBUTE)) {
+        crm_xml_replace(xml, F_ATTRD_ATTRIBUTE, NULL);
+    }
+    if (crm_element_value(xml, F_ATTRD_VALUE)) {
+        crm_xml_replace(xml, F_ATTRD_VALUE, NULL);
+    }
+
+    attrd_client_update(xml);
+}
+
 /*!
  * \internal
  * \brief Respond to a client refresh request (i.e. write out all attributes)
@@ -338,7 +363,7 @@ attrd_client_refresh(void)
     }
 
     crm_info("Updating all attributes");
-    write_attributes(TRUE, FALSE);
+    write_attributes(TRUE);
 }
 
 /*!
@@ -454,6 +479,50 @@ attrd_client_query(crm_client_t *client, uint32_t id, uint32_t flags, xmlNode *q
     free_xml(reply);
 }
 
+/*!
+ * \internal
+ * \brief Clear failure-related attributes
+ *
+ * \param[in] peer  Peer that sent clear request
+ * \param[in] xml   Request XML
+ */
+static void
+attrd_peer_clear_failure(crm_node_t *peer, xmlNode *xml)
+{
+    const char *rsc = crm_element_value(xml, F_ATTRD_RESOURCE);
+    const char *host = crm_element_value(xml, F_ATTRD_HOST);
+    const char *op = crm_element_value(xml, F_ATTRD_OPERATION);
+    const char *interval_s = crm_element_value(xml, F_ATTRD_INTERVAL);
+    int interval = crm_get_interval(interval_s);
+    char *attr = NULL;
+    GHashTableIter iter;
+    regex_t regex;
+
+    if (attrd_failure_regex(&regex, rsc, op, interval) != pcmk_ok) {
+        crm_info("Ignoring invalid request to clear failures for %s",
+                 (rsc? rsc : "all resources"));
+        return;
+    }
+
+    crm_xml_add(xml, F_ATTRD_TASK, ATTRD_OP_UPDATE);
+
+    /* Make sure value is not set, so we delete */
+    if (crm_element_value(xml, F_ATTRD_VALUE)) {
+        crm_xml_replace(xml, F_ATTRD_VALUE, NULL);
+    }
+
+    g_hash_table_iter_init(&iter, attributes);
+    while (g_hash_table_iter_next(&iter, (gpointer *) &attr, NULL)) {
+        if (regexec(&regex, attr, 0, NULL, 0) == 0) {
+            crm_trace("Matched %s when clearing %s",
+                      attr, (rsc? rsc : "all resources"));
+            crm_xml_add(xml, F_ATTRD_ATTRIBUTE, attr);
+            attrd_peer_update(peer, xml, host, FALSE);
+        }
+    }
+    regfree(&regex);
+}
+
 void
 attrd_peer_message(crm_node_t *peer, xmlNode *xml)
 {
@@ -541,6 +610,12 @@ attrd_peer_message(crm_node_t *peer, xmlNode *xml)
     } else if (safe_str_eq(op, ATTRD_OP_PEER_REMOVE)) {
         attrd_peer_remove(host, TRUE, peer->uname);
 
+    } else if (safe_str_eq(op, ATTRD_OP_CLEAR_FAILURE)) {
+        /* It is not currently possible to receive this as a peer command,
+         * but will be, if we one day enable propagating this operation.
+         */
+        attrd_peer_clear_failure(peer, xml);
+
     } else if (safe_str_eq(op, ATTRD_OP_SYNC_RESPONSE)
               && safe_str_neq(peer->uname, attrd_cluster->uname)) {
         xmlNode *child = NULL;
@@ -595,12 +670,12 @@ attrd_peer_remove(const char *host, gboolean uncache, const char *source)
     GHashTableIter aIter;
 
     CRM_CHECK(host != NULL, return);
-    crm_notice("Removing all %s attributes for %s", host, source);
+    crm_notice("Removing all %s attributes for peer %s", host, source);
 
     g_hash_table_iter_init(&aIter, attributes);
     while (g_hash_table_iter_next(&aIter, NULL, (gpointer *) & a)) {
         if(g_hash_table_remove(a->values, host)) {
-            crm_debug("Removed %s[%s] for %s", a->id, host, source);
+            crm_debug("Removed %s[%s] for peer %s", a->id, host, source);
         }
     }
 
@@ -657,124 +732,129 @@ attrd_lookup_or_create_value(GHashTable *values, const char *host, xmlNode *xml)
 void
 attrd_peer_update(crm_node_t *peer, xmlNode *xml, const char *host, bool filter)
 {
-    bool changed = FALSE;
+    bool update_both = FALSE;
     attribute_t *a;
     attribute_value_t *v = NULL;
-    int dampen = 0;
 
     const char *op = crm_element_value(xml, F_ATTRD_TASK);
     const char *attr = crm_element_value(xml, F_ATTRD_ATTRIBUTE);
     const char *value = crm_element_value(xml, F_ATTRD_VALUE);
-    const char *dvalue = crm_element_value(xml, F_ATTRD_DAMPEN);
 
     if (attr == NULL) {
-        crm_warn("Peer update did not specify attribute");
+        crm_warn("Could not update attribute: peer did not specify name");
         return;
     }
 
+    update_both = ((op == NULL) // ATTRD_OP_SYNC_RESPONSE has no F_ATTRD_TASK
+                   || safe_str_eq(op, ATTRD_OP_UPDATE_BOTH));
+
+    // Look up or create attribute entry
     a = g_hash_table_lookup(attributes, attr);
-    if(a == NULL) {
-        if (op == NULL /* The xml children from an ATTRD_OP_SYNC_RESPONSE have no F_ATTRD_TASK */
-            || safe_str_eq(op, ATTRD_OP_UPDATE)
-            || safe_str_eq(op, ATTRD_OP_UPDATE_BOTH)) {
+    if (a == NULL) {
+        if (update_both || safe_str_eq(op, ATTRD_OP_UPDATE)) {
             a = create_attribute(xml);
         } else {
-            crm_warn("Update error (attribute %s not found)", attr);
-            return;
-        }
-    }
-    
-    if (op == NULL /* The xml children from an ATTRD_OP_SYNC_RESPONSE have no F_ATTRD_TASK */
-        || safe_str_eq(op, ATTRD_OP_UPDATE_BOTH)
-        || safe_str_eq(op, ATTRD_OP_UPDATE_DELAY)) {
-        if (dvalue) {
-            dampen = crm_get_msec(dvalue); 
-            if (dampen >= 0) {
-                if (a->timeout_ms != dampen) {
-                    mainloop_timer_stop(a->timer);
-                    mainloop_timer_del(a->timer);
-                    a->timeout_ms = dampen;
-                    if (dampen > 0) {
-                        a->timer = mainloop_timer_add(a->id, a->timeout_ms, FALSE, attribute_timer_cb, a);
-                        crm_info("Update attribute %s with delay %dms (%s)", a->id, dampen, dvalue);
-                    } else {
-                        a->timer = NULL;
-                        crm_info("Update attribute %s with not delay", a->id);
-                    }
-                    //if dampen is changed, attrd writes in a current value immediately.
-                    write_or_elect_attribute(a);
-                    if (safe_str_eq(op, ATTRD_OP_UPDATE_DELAY)) {
-                        return;
-                    }
-                } else {
-                    if (safe_str_eq(op, ATTRD_OP_UPDATE_DELAY)) {
-                        crm_trace("Unchanged attribute %s with delay %dms (%s).(ATTRD_OP_UPDATE_DELAY)", a->id, dampen, dvalue);
-                        return;
-                    }
-                }
-            } else {
-                crm_warn("Update error (A positive number is necessary for delay parameter. attribute %s : %dms (%s))", a->id, dampen, dvalue);
-                return;
-            }
-        } else {
-            crm_warn("Update error (delay parameter is necessary for the update of the attribute %s)", a->id);
+            crm_warn("Could not update %s: attribute not found", attr);
             return;
         }
     }
 
-    if(host == NULL) {
+    // Update attribute dampening
+    if (update_both || safe_str_eq(op, ATTRD_OP_UPDATE_DELAY)) {
+        const char *dvalue = crm_element_value(xml, F_ATTRD_DAMPEN);
+        int dampen = 0;
+
+        if (dvalue == NULL) {
+            crm_warn("Could not update %s: peer did not specify value for delay",
+                     attr);
+            return;
+        }
+
+        dampen = crm_get_msec(dvalue);
+        if (dampen < 0) {
+            crm_warn("Could not update %s: invalid delay value %dms (%s)",
+                     attr, dampen, dvalue);
+            return;
+        }
+
+        if (a->timeout_ms != dampen) {
+            mainloop_timer_stop(a->timer);
+            mainloop_timer_del(a->timer);
+            a->timeout_ms = dampen;
+            if (dampen > 0) {
+                a->timer = mainloop_timer_add(attr, a->timeout_ms, FALSE,
+                                              attribute_timer_cb, a);
+                crm_info("Update attribute %s delay to %dms (%s)",
+                         attr, dampen, dvalue);
+            } else {
+                a->timer = NULL;
+                crm_info("Update attribute %s to remove delay", attr);
+            }
+
+            /* If dampening changed, do an immediate write-out,
+             * otherwise repeated dampening changes would prevent write-outs
+             */
+            write_or_elect_attribute(a);
+        }
+
+        if (!update_both) {
+            return;
+        }
+    }
+
+    // If no host was specified, update all hosts recursively
+    if (host == NULL) {
         GHashTableIter vIter;
-        g_hash_table_iter_init(&vIter, a->values);
 
         crm_debug("Setting %s for all hosts to %s", attr, value);
-
         xml_remove_prop(xml, F_ATTRD_HOST_ID);
+        g_hash_table_iter_init(&vIter, a->values);
         while (g_hash_table_iter_next(&vIter, (gpointer *) & host, NULL)) {
             attrd_peer_update(peer, xml, host, filter);
         }
         return;
     }
 
+    // Update attribute value for one host
+
     v = attrd_lookup_or_create_value(a->values, host, xml);
 
-    if(filter
-              && safe_str_neq(v->current, value)
-              && safe_str_eq(host, attrd_cluster->uname)) {
+    if (filter && safe_str_neq(v->current, value)
+        && safe_str_eq(host, attrd_cluster->uname)) {
+
         xmlNode *sync = create_xml_node(NULL, __FUNCTION__);
+
         crm_notice("%s[%s]: local value '%s' takes priority over '%s' from %s",
-                   a->id, host, v->current, value, peer->uname);
+                   attr, host, v->current, value, peer->uname);
 
         crm_xml_add(sync, F_ATTRD_TASK, ATTRD_OP_SYNC_RESPONSE);
         v = g_hash_table_lookup(a->values, host);
-        build_attribute_xml(sync, a->id, a->set, a->uuid, a->timeout_ms, a->user, a->is_private,
-                            v->nodename, v->nodeid, v->current);
+        build_attribute_xml(sync, attr, a->set, a->uuid, a->timeout_ms, a->user,
+                            a->is_private, v->nodename, v->nodeid, v->current);
 
         crm_xml_add_int(sync, F_ATTRD_WRITER, election_state(writer));
-        send_attrd_message(peer, sync);
+
+        /* Broadcast in case any other nodes had the inconsistent value */
+        send_attrd_message(NULL, sync);
         free_xml(sync);
 
-    } else if(safe_str_neq(v->current, value)) {
-        crm_info("Setting %s[%s]: %s -> %s from %s", attr, host, v->current, value, peer->uname);
+    } else if (safe_str_neq(v->current, value)) {
+        crm_info("Setting %s[%s]: %s -> %s from %s",
+                 attr, host, v->current, value, peer->uname);
         free(v->current);
-        if(value) {
-            v->current = strdup(value);
-        } else {
-            v->current = NULL;
-        }
-        changed = TRUE;
-    } else {
-        crm_trace("Unchanged %s[%s] from %s is %s", attr, host, peer->uname, value);
-    }
+        v->current = (value? strdup(value) : NULL);
+        a->changed = TRUE;
 
-    a->changed |= changed;
-
-    if(changed) {
-        if(a->timer) {
-            crm_trace("Delayed write out (%dms) for %s", a->timeout_ms, a->id);
+        // Write out new value or start dampening timer
+        if (a->timer) {
+            crm_trace("Delayed write out (%dms) for %s", a->timeout_ms, attr);
             mainloop_timer_start(a->timer);
         } else {
             write_or_elect_attribute(a);
         }
+
+    } else {
+        crm_trace("Unchanged %s[%s] from %s is %s", attr, host, peer->uname, value);
     }
 
     /* If this is a cluster node whose node ID we are learning, remember it */
@@ -783,11 +863,10 @@ attrd_peer_update(crm_node_t *peer, xmlNode *xml, const char *host, bool filter)
 
         crm_node_t *known_peer = crm_get_peer(v->nodeid, host);
 
-        crm_trace("We know %s's node id now: %s",
+        crm_trace("Learned %s has node id %s",
                   known_peer->uname, known_peer->uuid);
         if (election_state(writer) == election_won) {
-            write_attributes(FALSE, TRUE);
-            return;
+            write_attributes(FALSE);
         }
     }
 }
@@ -814,8 +893,6 @@ write_or_elect_attribute(attribute_t *a)
 gboolean
 attrd_election_cb(gpointer user_data)
 {
-    crm_trace("Election complete");
-
     free(peer_writer);
     peer_writer = strdup(attrd_cluster->uname);
 
@@ -823,7 +900,7 @@ attrd_election_cb(gpointer user_data)
     attrd_peer_sync(NULL, NULL);
 
     /* Update the CIB after an election */
-    write_attributes(TRUE, FALSE);
+    write_attributes(TRUE);
     return FALSE;
 }
 
@@ -842,7 +919,7 @@ attrd_peer_change_cb(enum crm_status_type kind, crm_node_t *peer, const void *da
             }
         } else {
             /* Remove all attribute values associated with lost nodes */
-            attrd_peer_remove(peer->uname, FALSE, "peer loss");
+            attrd_peer_remove(peer->uname, FALSE, "loss");
             if (peer_writer && safe_str_eq(peer->uname, peer_writer)) {
                 free(peer_writer);
                 peer_writer = NULL;
@@ -905,15 +982,16 @@ attrd_cib_callback(xmlNode * msg, int call_id, int rc, xmlNode * output, void *u
 }
 
 void
-write_attributes(bool all, bool peer_discovered)
+write_attributes(bool all)
 {
     GHashTableIter iter;
     attribute_t *a = NULL;
 
+    crm_debug("Writing out %s attributes", all? "all" : "changed");
     g_hash_table_iter_init(&iter, attributes);
     while (g_hash_table_iter_next(&iter, NULL, (gpointer *) & a)) {
-        if (peer_discovered && a->unknown_peer_uuids) {
-            /* a new peer uuid has been discovered, try writing this attribute again. */
+        if (!all && a->unknown_peer_uuids) {
+            // Try writing this attribute again, in case peer ID was learned
             a->changed = TRUE;
         }
 
@@ -928,31 +1006,9 @@ write_attributes(bool all, bool peer_discovered)
 static void
 build_update_element(xmlNode *parent, attribute_t *a, const char *nodeid, const char *value)
 {
-    char *set = NULL;
-    char *uuid = NULL;
+    const char *set = NULL;
     xmlNode *xml_obj = NULL;
 
-    if(a->set) {
-        set = strdup(a->set);
-    } else {
-        set = crm_strdup_printf("%s-%s", XML_CIB_TAG_STATUS, nodeid);
-    }
-
-    if(a->uuid) {
-        uuid = strdup(a->uuid);
-    } else {
-        int lpc;
-        uuid = crm_strdup_printf("%s-%s", set, a->id);
-
-        /* Minimal attempt at sanitizing automatic IDs */
-        for (lpc = 0; uuid[lpc] != 0; lpc++) {
-            switch (uuid[lpc]) {
-                case ':':
-                    uuid[lpc] = '.';
-            }
-        }
-    }
-
     xml_obj = create_xml_node(parent, XML_CIB_TAG_STATE);
     crm_xml_add(xml_obj, XML_ATTR_ID, nodeid);
 
@@ -960,10 +1016,19 @@ build_update_element(xmlNode *parent, attribute_t *a, const char *nodeid, const
     crm_xml_add(xml_obj, XML_ATTR_ID, nodeid);
 
     xml_obj = create_xml_node(xml_obj, XML_TAG_ATTR_SETS);
-    crm_xml_add(xml_obj, XML_ATTR_ID, set);
+    if (a->set) {
+        crm_xml_set_id(xml_obj, "%s", a->set);
+    } else {
+        crm_xml_set_id(xml_obj, "%s-%s", XML_CIB_TAG_STATUS, nodeid);
+    }
+    set = ID(xml_obj);
 
     xml_obj = create_xml_node(xml_obj, XML_CIB_TAG_NVPAIR);
-    crm_xml_add(xml_obj, XML_ATTR_ID, uuid);
+    if (a->uuid) {
+        crm_xml_set_id(xml_obj, "%s", a->uuid);
+    } else {
+        crm_xml_set_id(xml_obj, "%s-%s", set, a->id);
+    }
     crm_xml_add(xml_obj, XML_NVPAIR_ATTR_NAME, a->id);
 
     if(value) {
@@ -973,9 +1038,40 @@ build_update_element(xmlNode *parent, attribute_t *a, const char *nodeid, const
         crm_xml_add(xml_obj, XML_NVPAIR_ATTR_VALUE, "");
         crm_xml_add(xml_obj, "__delete__", XML_NVPAIR_ATTR_VALUE);
     }
+}
 
-    free(uuid);
-    free(set);
+static void
+set_alert_attribute_value(GHashTable *t, attribute_value_t *v)
+{
+    attribute_value_t *a_v = NULL;
+    a_v = calloc(1, sizeof(attribute_value_t));
+    CRM_ASSERT(a_v != NULL);
+
+    a_v->nodeid = v->nodeid;
+    a_v->nodename = strdup(v->nodename);
+
+    if (v->current != NULL) {
+        a_v->current = strdup(v->current);
+    }
+
+    g_hash_table_replace(t, a_v->nodename, a_v);
+}
+
+static void
+send_alert_attributes_value(attribute_t *a, GHashTable *t)
+{
+    int rc = 0;
+    attribute_value_t *at = NULL;
+    GHashTableIter vIter;
+
+    g_hash_table_iter_init(&vIter, t);
+
+    while (g_hash_table_iter_next(&vIter, NULL, (gpointer *) & at)) {
+        rc = attrd_send_attribute_alert(at->nodename, at->nodeid,
+                                        a->id, at->current);
+        crm_trace("Sent alerts for %s[%s]=%s: nodeid=%d rc=%d",
+                  a->id, at->nodename, at->current, at->nodeid, rc);
+    }
 }
 
 void
@@ -986,6 +1082,7 @@ write_attribute(attribute_t *a)
     attribute_value_t *v = NULL;
     GHashTableIter iter;
     enum cib_call_options flags = cib_quorum_override;
+    GHashTable *alert_attribute_value = NULL;
 
     if (a == NULL) {
         return;
@@ -1021,6 +1118,11 @@ write_attribute(attribute_t *a)
     /* We will check all peers' uuids shortly, so initialize this to false */
     a->unknown_peer_uuids = FALSE;
 
+    /* Make the table for the attribute trap */
+    alert_attribute_value = g_hash_table_new_full(crm_strcase_hash,
+                                                  crm_strcase_equal, NULL,
+                                                  free_attribute_value);
+
     /* Iterate over each peer value of this attribute */
     g_hash_table_iter_init(&iter, a->values);
     while (g_hash_table_iter_next(&iter, NULL, (gpointer *) & v)) {
@@ -1029,7 +1131,7 @@ write_attribute(attribute_t *a)
         /* If the value's peer info does not correspond to a peer, ignore it */
         if (peer == NULL) {
             crm_notice("Update error (peer not found): %s[%s]=%s failed (host=%p)",
-                       v->nodename, a->id, v->current, peer);
+                       a->id, v->nodename, v->current, peer);
             continue;
         }
 
@@ -1048,17 +1150,20 @@ write_attribute(attribute_t *a)
         /* If the peer is found, but its uuid is unknown, defer write */
         if (peer->uuid == NULL) {
             a->unknown_peer_uuids = TRUE;
-            crm_notice("Update error (unknown peer uuid, retry will be attempted once uuid is discovered): %s[%s]=%s failed (host=%p)",
-                       v->nodename, a->id, v->current, peer);
+            crm_notice("Update %s[%s]=%s postponed: unknown peer UUID, will retry if UUID is learned",
+                       a->id, v->nodename, v->current, peer);
             continue;
         }
 
         /* Add this value to status update XML */
-        crm_debug("Update: %s[%s]=%s (%s %u %u %s)", v->nodename, a->id,
+        crm_debug("Update: %s[%s]=%s (%s %u %u %s)", a->id, v->nodename,
                   v->current, peer->uuid, peer->id, v->nodeid, peer->uname);
         build_update_element(xml_top, a, peer->uuid, v->current);
         cib_updates++;
 
+        /* Preservation of the attribute to transmit alert */
+        set_alert_attribute_value(alert_attribute_value, v);
+
         free(v->requested);
         v->requested = NULL;
         if (v->current) {
@@ -1089,6 +1194,11 @@ write_attribute(attribute_t *a)
                                               strdup(a->id),
                                               "attrd_cib_callback",
                                               attrd_cib_callback, free);
+        /* Transmit alert of the attribute */
+        send_alert_attributes_value(a, alert_attribute_value);
+
     }
+
+    g_hash_table_destroy(alert_attribute_value);
     free_xml(xml_top);
 }
