@@ -125,7 +125,9 @@ peer_update_callback(enum crm_status_type type, crm_node_t * node, const void *d
     switch (type) {
         case crm_status_uname:
             /* If we've never seen the node, then it also won't be in the status section */
-            crm_info("%s is now %s", node->uname, state_text(node->state));
+            crm_info("%s node %s is now %s",
+                     (is_remote? "Remote" : "Cluster"),
+                     node->uname, state_text(node->state));
             return;
 
         case crm_status_rstate:
@@ -146,7 +148,7 @@ peer_update_callback(enum crm_status_type type, crm_node_t * node, const void *d
                 }
             }
 
-            crmd_notify_node_event(node);
+            crmd_alert_node_event(node);
             break;
 
         case crm_status_processes:
