@@ -18,34 +18,18 @@
 
 #include <crm_internal.h>
 
-#include <sys/param.h>
-#include <crm/crm.h>
-#include <crmd_fsa.h>
+#include <unistd.h>  /* sleep */
 
-#include <sys/types.h>
-#include <sys/wait.h>
-
-#include <unistd.h>             /* for access */
-
-#include <sys/types.h>          /* for calls to open */
-#include <sys/stat.h>           /* for calls to open */
-#include <fcntl.h>              /* for calls to open */
-#include <pwd.h>                /* for getpwuid */
-#include <grp.h>                /* for initgroups */
-
-#include <sys/time.h>           /* for getrlimit */
-#include <sys/resource.h>       /* for getrlimit */
-
-#include <errno.h>
-
-#include <crm/msg_xml.h>
+#include <crm/common/alerts_internal.h>
 #include <crm/common/xml.h>
-#include <crmd_messages.h>
-#include <crmd_callbacks.h>
+#include <crm/crm.h>
+#include <crm/msg_xml.h>
 
 #include <crmd.h>
+#include <crmd_callbacks.h>  /* crmd_cib_connection_destroy */
+#include <crmd_fsa.h>
+#include <crmd_messages.h>
 
-#include <tengine.h>
 
 struct crm_subsystem_s *cib_subsystem = NULL;
 
@@ -54,58 +38,8 @@ int cib_retries = 0;
 static void
 do_cib_updated(const char *event, xmlNode * msg)
 {
-    int rc = -1;
-    int format= 1;
-    xmlNode *patchset = get_message_xml(msg, F_CIB_UPDATE_RESULT);
-    xmlNode *change = NULL;
-    xmlXPathObject *xpathObj = NULL;
-
-    CRM_CHECK(msg != NULL, return);
-    crm_element_value_int(msg, F_CIB_RC, &rc);
-    if (rc < pcmk_ok) {
-        crm_trace("Filter rc=%d (%s)", rc, pcmk_strerror(rc));
-        return;
-    }
-
-    crm_element_value_int(patchset, "format", &format);
-    if (format == 1) {
-        if ((xpathObj = xpath_search(
-                 msg,
-                 "//" F_CIB_UPDATE_RESULT "//" XML_TAG_DIFF_ADDED "//" XML_CIB_TAG_CRMCONFIG " | " \
-                 "//" F_CIB_UPDATE_RESULT "//" XML_TAG_DIFF_ADDED "//" XML_CIB_TAG_ALERTS
-                 )) != NULL) {
-            freeXpathObject(xpathObj);
-            mainloop_set_trigger(config_read);
-        }
-    } else if (format == 2) {
-        for (change = __xml_first_child(patchset); change != NULL; change = __xml_next(change)) {
-            const char *xpath = crm_element_value(change, XML_DIFF_PATH);
-
-            if (xpath == NULL) {
-                continue;
-            }
-
-            /* modifying properties */
-            if (!strstr(xpath, "/" XML_TAG_CIB "/" XML_CIB_TAG_CONFIGURATION "/" XML_CIB_TAG_CRMCONFIG "/") &&
-                !strstr(xpath, "/" XML_TAG_CIB "/" XML_CIB_TAG_CONFIGURATION "/" XML_CIB_TAG_ALERTS)) {
-                xmlNode *section = NULL;
-                const char *name = NULL;
-
-                /* adding notifications section */
-                if ((strcmp(xpath, "/" XML_TAG_CIB "/" XML_CIB_TAG_CONFIGURATION) != 0) ||
-                    ((section = __xml_first_child(change)) == NULL) ||
-                    ((name = crm_element_name(section)) == NULL) ||
-                    (strcmp(name, XML_CIB_TAG_ALERTS) != 0)) {
-                    continue;
-                }
-            } 
-
-            mainloop_set_trigger(config_read);
-            break;
-        }
-
-    } else {
-        crm_warn("Unknown patch format: %d", format);
+    if (crm_patchset_contains_alert(msg, TRUE)) {
+        mainloop_set_trigger(config_read);
     }
 }
 
@@ -127,7 +61,7 @@ revision_check_callback(xmlNode * msg, int call_id, int rc, xmlNode * output, vo
     CRM_CHECK(safe_str_eq(crm_element_name(generation), XML_TAG_CIB),
               crm_log_xml_err(output, __FUNCTION__); return);
 
-    crm_trace("Checking our feature revision is allowed: %s", CIB_FEATURE_SET);
+    crm_trace("Checking our feature revision %s is allowed", CRM_FEATURE_SET);
 
     revision = crm_element_value(generation, XML_ATTR_CRM_VERSION);
     cmp = compare_version(revision, CRM_FEATURE_SET);
@@ -159,19 +93,16 @@ do_cib_replaced(const char *event, xmlNode * msg)
     register_fsa_input(C_FSA_INTERNAL, I_ELECTION, NULL);
 }
 
-/*	 A_CIB_STOP, A_CIB_START, A_CIB_RESTART,	*/
+/* A_CIB_STOP, A_CIB_START, O_CIB_RESTART */
 void
 do_cib_control(long long action,
                enum crmd_fsa_cause cause,
                enum crmd_fsa_state cur_state,
                enum crmd_fsa_input current_input, fsa_data_t * msg_data)
 {
-    struct crm_subsystem_s *this_subsys = cib_subsystem;
+    CRM_ASSERT(fsa_cib_conn != NULL);
 
-    long long stop_actions = A_CIB_STOP;
-    long long start_actions = A_CIB_START;
-
-    if (action & stop_actions) {
+    if (action & A_CIB_STOP) {
 
         if (fsa_cib_conn->state != cib_disconnected && last_resource_update != 0) {
             crm_info("Waiting for resource update %d to complete", last_resource_update);
@@ -181,7 +112,6 @@ do_cib_control(long long action,
 
         crm_info("Disconnecting CIB");
         clear_bit(fsa_input_register, R_CIB_CONNECTED);
-        CRM_ASSERT(fsa_cib_conn != NULL);
 
         fsa_cib_conn->cmds->del_notify_callback(fsa_cib_conn, T_CIB_DIFF_NOTIFY, do_cib_updated);
 
@@ -189,15 +119,14 @@ do_cib_control(long long action,
             fsa_cib_conn->cmds->set_slave(fsa_cib_conn, cib_scope_local);
             fsa_cib_conn->cmds->signoff(fsa_cib_conn);
         }
+        crm_notice("Disconnected from the CIB");
     }
 
-    if (action & start_actions) {
+    if (action & A_CIB_START) {
         int rc = pcmk_ok;
 
-        CRM_ASSERT(fsa_cib_conn != NULL);
-
         if (cur_state == S_STOPPING) {
-            crm_err("Ignoring request to start %s after shutdown", this_subsys->name);
+            crm_err("Ignoring request to start the CIB after shutdown");
             return;
         }
 
