@@ -19,9 +19,10 @@
 #  define CRMD_UTILS__H
 
 #  include <crm/crm.h>
+#  include <crm/transition.h>
 #  include <crm/common/xml.h>
 #  include <crm/cib/internal.h> /* For CIB_OP_MODIFY */
-#  include "notify.h"
+#  include "crmd_alerts.h"
 
 #  define CLIENT_EXIT_WAIT 30
 #  define FAKE_TE_ID	"xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx"
@@ -92,14 +93,20 @@ void crm_update_quorum(gboolean quorum, gboolean force_update);
 void erase_status_tag(const char *uname, const char *tag, int options);
 void update_attrd(const char *host, const char *name, const char *value, const char *user_name, gboolean is_remote_node);
 void update_attrd_remote_node_removed(const char *host, const char *user_name);
+void update_attrd_clear_failures(const char *host, const char *rsc,
+                                 const char *op, const char *interval,
+                                 gboolean is_remote_node);
 
 int crmd_join_phase_count(enum crm_join_phase phase);
 void crmd_join_phase_log(int level);
 
 const char *get_timer_desc(fsa_timer_t * timer);
-gboolean too_many_st_failures(void);
 void st_fail_count_reset(const char * target);
+void st_fail_count_increment(const char *target);
+void abort_for_stonith_failure(enum transition_action abort_action,
+                               const char *target, xmlNode *reason);
 void crmd_peer_down(crm_node_t *peer, bool full);
+unsigned int cib_op_timeout(void);
 
 /* Convenience macro for registering a CIB callback
  * (assumes that data can be freed with free())
@@ -107,7 +114,7 @@ void crmd_peer_down(crm_node_t *peer, bool full);
 #  define fsa_register_cib_callback(id, flag, data, fn) do {            \
     CRM_ASSERT(fsa_cib_conn);                                           \
     fsa_cib_conn->cmds->register_callback_full(                         \
-            fsa_cib_conn, id, 10 * (1 + crm_active_peers()),            \
+        fsa_cib_conn, id, cib_op_timeout(),                             \
             flag, data, #fn, fn, free);                                 \
     } while(0)
 
