@@ -354,7 +354,7 @@ crmd_ccm_msg_callback(oc_ed_t event, void *cookie, size_t size, const void *data
     if (update_quorum) {
         crm_have_quorum = ccm_have_quorum(event);
         if (crm_have_quorum == FALSE) {
-            /* did we just loose quorum? */
+            /* did we just lose quorum? */
             if (fsa_has_quorum) {
                 crm_info("Quorum lost: %s", ccm_event_name(event));
             }
@@ -403,7 +403,8 @@ crmd_ha_status_callback(const char *node, const char *status, void *private)
     trigger_fsa(fsa_source);
 
     if (AM_I_DC) {
-        update = do_update_node_cib(peer, node_update_cluster, NULL, __FUNCTION__);
+        update = create_node_state_update(peer, node_update_cluster, NULL,
+                                          __FUNCTION__);
         fsa_cib_anon_update(XML_CIB_TAG_STATUS, update,
                             cib_scope_local | cib_quorum_override | cib_can_create);
         free_xml(update);
@@ -456,7 +457,8 @@ crmd_client_status_callback(const char *node, const char *client, const char *st
         xmlNode *update = NULL;
 
         crm_trace("Got client status callback");
-        update = do_update_node_cib(peer, node_update_peer, NULL, __FUNCTION__);
+        update = create_node_state_update(peer, node_update_peer, NULL,
+                                          __FUNCTION__);
         fsa_cib_anon_update(XML_CIB_TAG_STATUS, update,
                             cib_scope_local | cib_quorum_override | cib_can_create);
         free_xml(update);
@@ -530,7 +532,7 @@ crmd_ha_msg_dispatch(ll_cluster_t * cluster_conn, gpointer user_data)
         if (cluster_conn->llc_ops->msgready(cluster_conn) == 0) {
             crm_trace("no message ready yet");
         }
-        /* invoke the callbacks but dont block */
+        /* invoke the callbacks but don't block */
         cluster_conn->llc_ops->rcvmsg(cluster_conn, 0);
     }
 
