@@ -361,18 +361,15 @@ class Environment:
                 elif args[i+1] == "phd":
                     self["DoStonith"]=1
                     self["stonith-type"] = "fence_phd_kvm"
-                    self["stonith-params"] = "pcmk_arg_map=domain:uname,delay=0"
                 elif args[i+1] == "rhcs" or args[i+1] == "xvm" or args[i+1] == "virt":
                     self["DoStonith"]=1
                     self["stonith-type"] = "fence_xvm"
-                    self["stonith-params"] = "pcmk_arg_map=domain:uname,delay=0"
                 elif args[i+1] == "docker":
                     self["DoStonith"]=1
                     self["stonith-type"] = "fence_docker_cts"
                 elif args[i+1] == "scsi":
                     self["DoStonith"]=1
                     self["stonith-type"] = "fence_scsi"
-                    self["stonith-params"] = "delay=0"
                 elif args[i+1] == "ssh" or args[i+1] == "lha":
                     self["DoStonith"]=1
                     self["stonith-type"] = "external/ssh"
@@ -670,10 +667,10 @@ class Environment:
         print("\t [--stonith-type type]")
         print("\t [--stonith-args name=value]")
         print("\t [--bsc]")
-        print("\t [--notification-agent path]  script to configure for Pacemaker notifications")
-        print("\t [--notification-recipient r] recipient to pass to notification agent")
-        print("\t [--no-loop-tests]            dont run looping/time-based tests")
-        print("\t [--no-unsafe-tests]          dont run tests that are unsafe for use with ocfs2/drbd")
+        print("\t [--notification-agent path]  script to configure for Pacemaker alerts")
+        print("\t [--notification-recipient r] recipient to pass to alert script")
+        print("\t [--no-loop-tests]            don't run looping/time-based tests")
+        print("\t [--no-unsafe-tests]          don't run tests that are unsafe for use with ocfs2/drbd")
         print("\t [--valgrind-tests]           include tests using valgrind")
         print("\t [--experimental-tests]       include experimental tests")
         print("\t [--container-tests]          include pacemaker_remote tests that run in lxc container resources")
