@@ -37,13 +37,13 @@
 #  include <libxml/tree.h>
 #  include <libxml/xpath.h>
 
-/* Compression costs a LOT, don't do it unless we're hitting message limits
+/* Define compression parameters for IPC messages
  *
- * For now, use 256k as the lower size, which means we can have 4 big data fields
- *  before we hit heartbeat's message limit
- *
- * The previous limit was 10k, compressing 184 of 1071 messages accounted for 23%
- *  of the total CPU used by the cib
+ * Compression costs a LOT, so we don't want to do it unless we're hitting
+ * message limits. Currently, we use 128KB as the threshold, because higher
+ * values don't play well with the heartbeat stack. With an earlier limit of
+ * 10KB, compressing 184 of 1071 messages accounted for 23% of the total CPU
+ * used by the cib.
  */
 #  define CRM_BZ2_BLOCKS		4
 #  define CRM_BZ2_WORK		20
@@ -55,6 +55,9 @@ gboolean add_message_xml(xmlNode * msg, const char *field, xmlNode * xml);
 xmlNode *get_message_xml(xmlNode * msg, const char *field);
 GHashTable *xml2list(xmlNode * parent);
 
+xmlNode *crm_create_nvpair_xml(xmlNode *parent, const char *id,
+                               const char *name, const char *value);
+
 void hash2nvpair(gpointer key, gpointer value, gpointer user_data);
 void hash2field(gpointer key, gpointer value, gpointer user_data);
 void hash2metafield(gpointer key, gpointer value, gpointer user_data);
@@ -107,7 +110,7 @@ const char *crm_xml_add_int(xmlNode * node, const char *name, int value);
  * Add an attribute with the value XML_BOOLEAN_TRUE or XML_BOOLEAN_FALSE
  * as appropriate to an XML object.
  *
- * \param[in/out] node   XML object to add attribute to
+ * \param[in,out] node   XML object to add attribute to
  * \param[in]     name   Name of attribute to add
  * \param[in]     value  Boolean whose value will be tested
  *
@@ -210,6 +213,24 @@ crm_element_name(xmlNode *xml)
 
 const char *crm_element_value(xmlNode * data, const char *name);
 
+/*!
+ * \brief Copy an element from one XML object to another
+ *
+ * \param[in]     obj1     Source XML
+ * \param[in,out] obj2     Destination XML
+ * \param[in]     element  Name of element to copy
+ *
+ * \return Pointer to copied value (from source)
+ */
+static inline const char *
+crm_copy_xml_element(xmlNode *obj1, xmlNode *obj2, const char *element)
+{
+    const char *value = crm_element_value(obj1, element);
+
+    crm_xml_add(obj2, element, value);
+    return value;
+}
+
 void xml_validate(const xmlNode * root);
 
 gboolean xml_has_children(const xmlNode * root);
@@ -224,7 +245,7 @@ gboolean validate_xml(xmlNode * xml_blob, const char *validation, gboolean to_lo
 gboolean validate_xml_verbose(xmlNode * xml_blob);
 
 /*!
- * \brief Try update CIB XML to the highest pacemaker's standard if feasible
+ * \brief Update CIB XML to most recent schema version
  *
  * "Update" means either actively employ XSLT-based transformation(s)
  * (if intermediate product to transform valid per its declared schema version,
@@ -234,7 +255,7 @@ gboolean validate_xml_verbose(xmlNode * xml_blob);
  * such attempt subsequently fails to validate.   Which of the two styles will
  * be used depends on \p transform parameter (positive/negative, respectively).
  *
- * \param[in/out] xml_blob   XML tree representing CIB, may be swapped with
+ * \param[in,out] xml_blob   XML tree representing CIB, may be swapped with
  *                           an "updated" one
  * \param[out]    best       The highest configuration version (per its index
  *                           in the global schemas table) it was possible to
@@ -297,21 +318,40 @@ __xml_next(xmlNode * child)
     return child;
 }
 
+static inline xmlNode *
+__xml_first_child_element(xmlNode * parent)
+{
+    xmlNode *child = NULL;
+
+    if (parent) {
+        child = parent->children;
+    }
+
+    while (child) {
+        if(child->type == XML_ELEMENT_NODE) {
+            return child;
+        }
+        child = child->next;
+    }
+    return NULL;
+}
+
 static inline xmlNode *
 __xml_next_element(xmlNode * child)
 {
-    if (child) {
+    while (child) {
         child = child->next;
-        while (child && child->type != XML_ELEMENT_NODE) {
-            child = child->next;
+        if(child && child->type == XML_ELEMENT_NODE) {
+            return child;
         }
     }
-    return child;
+    return NULL;
 }
 
 void free_xml(xmlNode * child);
 
 xmlNode *first_named_child(xmlNode * parent, const char *name);
+xmlNode *crm_next_same_xml(xmlNode *sibling);
 
 xmlNode *sorted_xml(xmlNode * input, xmlNode * parent, gboolean recursive);
 xmlXPathObjectPtr xpath_search(xmlNode * xml_top, const char *path);
@@ -355,4 +395,13 @@ void save_xml_to_file(xmlNode * xml, const char *desc, const char *filename);
 char *xml_get_path(xmlNode *xml);
 
 char * crm_xml_escape(const char *text);
+void crm_xml_sanitize_id(char *id);
+void crm_xml_set_id(xmlNode *xml, const char *format, ...)
+    __attribute__ ((__format__ (__printf__, 2, 3)));
+
+/*!
+ * \brief xmlNode destructor which can be used in glib collections
+ */
+void crm_destroy_xml(gpointer data);
+
 #endif
