@@ -223,6 +223,8 @@ typedef union
 #  endif
 
 /* Replacement error codes for non-linux */
+#  include <errno.h>
+
 #  ifndef ENOTUNIQ
 #    define ENOTUNIQ  190
 #  endif
@@ -263,6 +265,10 @@ typedef union
 #    define ENOSTR    199
 #  endif
 
+#  ifndef EKEYREJECTED
+#    define EKEYREJECTED 200
+#  endif
+
 /*
  * Some compilers (eg. Sun studio) do not define __FUNCTION__
  */
