@@ -32,6 +32,8 @@
 #include <crm/msg_xml.h>
 #include <crm/stonith-ng.h>
 
+#define s_if_plural(i) (((i) == 1)? "" : "s")
+
 /* The peer cache remembers cluster nodes that have been seen.
  * This is managed mostly automatically by libcluster, based on
  * cluster membership events.
@@ -256,6 +258,8 @@ crm_remote_peer_cache_refresh(xmlNode *cib)
 {
     struct refresh_data data;
 
+    crm_peer_init();
+
     /* First, we mark all existing cache entries as dirty,
      * so that later we can remove any that weren't in the CIB.
      * We don't empty the cache, because we need to detect changes in state.
@@ -359,8 +363,8 @@ reap_crm_member(uint32_t id, const char *name)
     search.uname = name ? strdup(name) : NULL;
     matches = g_hash_table_foreach_remove(crm_peer_cache, crm_reap_dead_member, &search);
     if(matches) {
-        crm_notice("Purged %d peers with id=%u%s%s from the membership cache",
-                   matches, search.id,
+        crm_notice("Purged %d peer%s with id=%u%s%s from the membership cache",
+                   matches, s_if_plural(matches), search.id,
                    (search.uname? " and/or uname=" : ""),
                    (search.uname? search.uname : ""));
 
@@ -833,10 +837,19 @@ crm_update_peer(const char *source, unsigned int id, uint64_t born, uint64_t see
 void
 crm_update_peer_uname(crm_node_t *node, const char *uname)
 {
-    int i, len = strlen(uname);
+    CRM_CHECK(uname != NULL,
+              crm_err("Bug: can't update node name without name"); return);
+    CRM_CHECK(node != NULL,
+              crm_err("Bug: can't update node name to %s without node", uname);
+              return);
 
-    for (i = 0; i < len; i++) {
-        if (uname[i] >= 'A' && uname[i] <= 'Z') {
+    if (safe_str_eq(uname, node->uname)) {
+        crm_debug("Node uname '%s' did not change", uname);
+        return;
+    }
+
+    for (const char *c = uname; *c; ++c) {
+        if ((*c >= 'A') && (*c <= 'Z')) {
             crm_warn("Node names with capitals are discouraged, consider changing '%s'",
                      uname);
             break;
@@ -845,6 +858,8 @@ crm_update_peer_uname(crm_node_t *node, const char *uname)
 
     free(node->uname);
     node->uname = strdup(uname);
+    CRM_ASSERT(node->uname != NULL);
+
     if (crm_status_callback) {
         crm_status_callback(crm_status_uname, node, NULL);
     }
