@@ -27,8 +27,6 @@
 #define VARIANT_CLONE 1
 #include "./variant.h"
 
-void clone_create_notifications(resource_t * rsc, action_t * action, action_t * action_complete,
-                                pe_working_set_t * data_set);
 void force_non_unique_clone(resource_t * rsc, const char *rid, pe_working_set_t * data_set);
 resource_t *create_child_clone(resource_t * rsc, int sub_id, pe_working_set_t * data_set);
 
@@ -49,7 +47,7 @@ mark_as_orphan(resource_t * rsc)
 void
 force_non_unique_clone(resource_t * rsc, const char *rid, pe_working_set_t * data_set)
 {
-    if (rsc->variant == pe_clone || rsc->variant == pe_master) {
+    if (pe_rsc_is_clone(rsc)) {
         clone_variant_data_t *clone_data = NULL;
 
         get_clone_variant_data(clone_data, rsc);
@@ -155,8 +153,6 @@ gboolean
 clone_unpack(resource_t * rsc, pe_working_set_t * data_set)
 {
     int lpc = 0;
-    const char *type = NULL;
-    int num_xml_children = 0;
     xmlNode *a_child = NULL;
     xmlNode *xml_obj = rsc->xml;
     clone_variant_data_t *clone_data = NULL;
@@ -170,8 +166,6 @@ clone_unpack(resource_t * rsc, pe_working_set_t * data_set)
 
     clone_data = calloc(1, sizeof(clone_variant_data_t));
     rsc->variant_opaque = clone_data;
-    clone_data->interleave = FALSE;
-    clone_data->ordered = FALSE;
 
     clone_data->active_clones = 0;
     clone_data->xml_obj_child = NULL;
@@ -187,12 +181,9 @@ clone_unpack(resource_t * rsc, pe_working_set_t * data_set)
         clone_data->clone_max = 1;      /* Handy during crm_verify */
     }
 
-    if (crm_is_true(interleave)) {
-        clone_data->interleave = TRUE;
-    }
-    if (crm_is_true(ordered)) {
-        clone_data->ordered = TRUE;
-    }
+    clone_data->interleave = crm_is_true(interleave);
+    clone_data->ordered = crm_is_true(ordered);
+
     if ((rsc->flags & pe_rsc_unique) == 0 && clone_data->clone_node_max > 1) {
         crm_config_err("Anonymous clones (%s) may only support one copy per node", rsc->id);
         clone_data->clone_node_max = 1;
@@ -204,14 +195,14 @@ clone_unpack(resource_t * rsc, pe_working_set_t * data_set)
     pe_rsc_trace(rsc, "\tClone is unique: %s",
                  is_set(rsc->flags, pe_rsc_unique) ? "true" : "false");
 
-    clone_data->xml_obj_child = find_xml_node(xml_obj, XML_CIB_TAG_GROUP, FALSE);
+    // Clones may contain a single group or primitive
+    for (a_child = __xml_first_child(xml_obj); a_child != NULL;
+         a_child = __xml_next_element(a_child)) {
 
-    if (clone_data->xml_obj_child == NULL) {
-        clone_data->xml_obj_child = find_xml_node(xml_obj, XML_CIB_TAG_RESOURCE, TRUE);
-        for (a_child = __xml_first_child(xml_obj); a_child != NULL; a_child = __xml_next_element(a_child)) {
-            if (crm_str_eq((const char *)a_child->name, XML_CIB_TAG_RESOURCE, TRUE)) {
-                num_xml_children++;
-            }
+        if (crm_str_eq((const char *)a_child->name, XML_CIB_TAG_RESOURCE, TRUE)
+        || crm_str_eq((const char *)a_child->name, XML_CIB_TAG_GROUP, TRUE)) {
+            clone_data->xml_obj_child = a_child;
+            break;
         }
     }
 
@@ -220,17 +211,6 @@ clone_unpack(resource_t * rsc, pe_working_set_t * data_set)
         return FALSE;
     }
 
-    for (a_child = __xml_first_child(xml_obj); a_child != NULL; a_child = __xml_next_element(a_child)) {
-        if (crm_str_eq((const char *)a_child->name, type, TRUE)) {
-            num_xml_children++;
-        }
-    }
-
-    if (num_xml_children > 1) {
-        crm_config_err("%s has too many children.  Only the first (%s) will be cloned.",
-                       rsc->id, ID(clone_data->xml_obj_child));
-    }
-
     /*
      * Make clones ever so slightly sticky by default
      *
@@ -314,11 +294,36 @@ short_print(char *list, const char *prefix, const char *type, const char *suffix
     }
 }
 
+static const char *
+configured_role_str(resource_t * rsc)
+{
+    const char *target_role = g_hash_table_lookup(rsc->meta,
+                                                  XML_RSC_ATTR_TARGET_ROLE);
+
+    if ((target_role == NULL) && rsc->children && rsc->children->data) {
+        target_role = g_hash_table_lookup(((resource_t*)rsc->children->data)->meta,
+                                          XML_RSC_ATTR_TARGET_ROLE);
+    }
+    return target_role;
+}
+
+static enum rsc_role_e
+configured_role(resource_t * rsc)
+{
+    const char *target_role = configured_role_str(rsc);
+
+    if (target_role) {
+        return text2role(target_role);
+    }
+    return RSC_ROLE_UNKNOWN;
+}
+
 static void
 clone_print_xml(resource_t * rsc, const char *pre_text, long options, void *print_data)
 {
     int is_master_slave = rsc->variant == pe_master ? 1 : 0;
     char *child_text = crm_concat(pre_text, "   ", ' ');
+    const char *target_role = configured_role_str(rsc);
     GListPtr gIter = rsc->children;
 
     status_print("%s<clone ", pre_text);
@@ -329,6 +334,9 @@ clone_print_xml(resource_t * rsc, const char *pre_text, long options, void *prin
     status_print("failed=\"%s\" ", is_set(rsc->flags, pe_rsc_failed) ? "true" : "false");
     status_print("failure_ignored=\"%s\" ",
                  is_set(rsc->flags, pe_rsc_failure_ignored) ? "true" : "false");
+    if (target_role) {
+        status_print("target_role=\"%s\" ", target_role);
+    }
     status_print(">\n");
 
     for (; gIter != NULL; gIter = gIter->next) {
@@ -371,21 +379,6 @@ bool is_set_recursive(resource_t * rsc, long long flag, bool any)
     return FALSE;
 }
 
-static enum rsc_role_e
-configured_role(resource_t * rsc)
-{
-    const char *target_role = g_hash_table_lookup(rsc->meta, XML_RSC_ATTR_TARGET_ROLE);
-
-    if(target_role == NULL) {
-        target_role = g_hash_table_lookup(((resource_t*)rsc->children->data)->meta, XML_RSC_ATTR_TARGET_ROLE);
-    }
-
-    if(target_role) {
-        return text2role(target_role);
-    }
-    return RSC_ROLE_UNKNOWN;
-}
-
 void
 clone_print(resource_t * rsc, const char *pre_text, long options, void *print_data)
 {
