@@ -307,7 +307,7 @@ start_child(pcmk_child_t * child)
             opts_vgrind[3] = NULL;
             opts_vgrind[4] = NULL;
         }
-        opts_default[0] = strdup(child->command);;
+        opts_default[0] = strdup(child->command);
 
         if(gid) {
             if(stack == pcmk_cluster_corosync) {
@@ -384,7 +384,7 @@ pcmk_shutdown_worker(gpointer user_data)
     }
 
     for (; phase > 0; phase--) {
-        /* dont stop anything with start_seq < 1 */
+        /* Don't stop anything with start_seq < 1 */
 
         for (lpc = max - 1; lpc >= 0; lpc--) {
             pcmk_child_t *child = &(pcmk_children[lpc]);
@@ -606,13 +606,11 @@ update_process_peers(void)
     struct iovec *iov;
     int rc = 0;
 
-    memset(buffer, 0, SIZEOF(buffer));
-
     if (local_name) {
-        rc = snprintf(buffer, SIZEOF(buffer) - 1, "<node uname=\"%s\" proclist=\"%u\"/>",
+        rc = snprintf(buffer, SIZEOF(buffer), "<node uname=\"%s\" proclist=\"%u\"/>",
                       local_name, get_process_list());
     } else {
-        rc = snprintf(buffer, SIZEOF(buffer) - 1, "<node proclist=\"%u\"/>", get_process_list());
+        rc = snprintf(buffer, SIZEOF(buffer), "<node proclist=\"%u\"/>", get_process_list());
     }
 
     crm_trace("Sending %s", buffer);
@@ -671,6 +669,7 @@ static struct crm_option long_options[] = {
     {"-spacer-",       1, 0, '-', "\nAdditional Options:"},
     {"foreground",     0, 0, 'f', "\t(Ignored) Pacemaker always runs in the foreground"},
     {"pid-file",       1, 0, 'p', "\t(Ignored) Daemon pid file location"},
+    {"standby",        0, 0, 's', "\tStart node in standby state"},
 
     {NULL, 0, 0, 0}
 };
@@ -782,7 +781,7 @@ init_children_processes(void)
 
     /* start any children that have not been detected */
     for (start_seq = 1; start_seq < max; start_seq++) {
-        /* dont start anything with start_seq < 1 */
+        /* don't start anything with start_seq < 1 */
         for (lpc = 0; lpc < max; lpc++) {
             if (pcmk_children[lpc].pid) {
                 /* we are already tracking it */
@@ -941,6 +940,9 @@ main(int argc, char **argv)
             case 'p':
                 pid_file = optarg;
                 break;
+            case 's':
+                set_daemon_option("node_start_state", "standby");
+                break;
             case '$':
             case '?':
                 crm_help(flag, EX_OK);
@@ -1058,20 +1060,24 @@ main(int argc, char **argv)
     mkdir(CRM_STATE_DIR, 0750);
     mcp_chown(CRM_STATE_DIR, pcmk_uid, pcmk_gid);
 
+    /* Used to store core/blackbox/pengine/cib files in */
+    crm_build_path(CRM_PACEMAKER_DIR, 0750);
+    mcp_chown(CRM_PACEMAKER_DIR, pcmk_uid, pcmk_gid);
+
     /* Used to store core files in */
-    crm_build_path(CRM_CORE_DIR, 0775);
+    crm_build_path(CRM_CORE_DIR, 0750);
     mcp_chown(CRM_CORE_DIR, pcmk_uid, pcmk_gid);
 
     /* Used to store blackbox dumps in */
-    crm_build_path(CRM_BLACKBOX_DIR, 0755);
+    crm_build_path(CRM_BLACKBOX_DIR, 0750);
     mcp_chown(CRM_BLACKBOX_DIR, pcmk_uid, pcmk_gid);
 
     /* Used to store policy engine inputs in */
-    crm_build_path(PE_STATE_DIR, 0755);
+    crm_build_path(PE_STATE_DIR, 0750);
     mcp_chown(PE_STATE_DIR, pcmk_uid, pcmk_gid);
 
     /* Used to store the cluster configuration */
-    crm_build_path(CRM_CONFIG_DIR, 0755);
+    crm_build_path(CRM_CONFIG_DIR, 0750);
     mcp_chown(CRM_CONFIG_DIR, pcmk_uid, pcmk_gid);
 
     /* Resource agent paths are constructed by the lrmd */
