@@ -54,7 +54,7 @@ enum pe_ordering get_flags(const char *id, enum pe_order_kind kind,
 enum pe_ordering get_asymmetrical_flags(enum pe_order_kind kind);
 static rsc_to_node_t *generate_location_rule(resource_t * rsc, xmlNode * rule_xml,
                                              const char *discovery, pe_working_set_t * data_set,
-                                             pe_re_match_data_t * match_data);
+                                             pe_match_data_t * match_data);
 
 gboolean
 unpack_constraints(xmlNode * xml_constraints, pe_working_set_t * data_set)
@@ -153,7 +153,7 @@ get_ordering_type(xmlNode * xml_obj)
                 kind_e = pe_order_kind_optional;
             }
 
-            /* } else if(rsc_then->variant == pe_native && rsc_first->variant > pe_group) { */
+            /* } else if(rsc_then->variant == pe_native && rsc_first->variant >= pe_clone) { */
             /*     kind_e = pe_order_kind_optional; */
         }
 
@@ -181,9 +181,8 @@ pe_find_constraint_resource(GListPtr rsc_list, const char *id)
 
     for (rIter = rsc_list; id && rIter; rIter = rIter->next) {
         resource_t *parent = rIter->data;
-
-        resource_t *match =
-            parent->fns->find_rsc(parent, id, NULL, pe_find_renamed | pe_find_current);
+        resource_t *match = parent->fns->find_rsc(parent, id, NULL,
+                                                  pe_find_renamed);
 
         if (match != NULL) {
             if(safe_str_neq(match->id, id)) {
@@ -316,13 +315,13 @@ unpack_simple_rsc_order(xmlNode * xml_obj, pe_working_set_t * data_set)
         crm_config_err("Constraint %s: no resource found for name '%s'", id, id_first);
         return FALSE;
 
-    } else if (instance_then && rsc_then->variant < pe_clone) {
+    } else if (instance_then && pe_rsc_is_clone(rsc_then) == FALSE) {
         crm_config_err("Invalid constraint '%s':"
                        " Resource '%s' is not a clone but instance %s was requested",
                        id, id_then, instance_then);
         return FALSE;
 
-    } else if (instance_first && rsc_first->variant < pe_clone) {
+    } else if (instance_first && pe_rsc_is_clone(rsc_first) == FALSE) {
         crm_config_err("Invalid constraint '%s':"
                        " Resource '%s' is not a clone but instance %s was requested",
                        id, id_first, instance_first);
@@ -350,11 +349,11 @@ unpack_simple_rsc_order(xmlNode * xml_obj, pe_working_set_t * data_set)
     require_all_s = crm_element_value(xml_obj, "require-all");
     if (require_all_s
         && crm_is_true(require_all_s) == FALSE
-        && rsc_first->variant >= pe_clone) {
+        && pe_rsc_is_clone(rsc_first)) {
 
         /* require-all=false means only one instance of the clone is required */
         min_required_before = 1;
-    } else if (rsc_first->variant >= pe_clone) {
+    } else if (pe_rsc_is_clone(rsc_first)) {
         const char *min_clones_s = g_hash_table_lookup(rsc_first->meta, XML_RSC_ATTR_INCARNATION_MIN);
         if (min_clones_s) {
             /* if clone min is set, we require at a minimum X number of instances
@@ -390,7 +389,7 @@ unpack_simple_rsc_order(xmlNode * xml_obj, pe_working_set_t * data_set)
          * actions to be considered runnable before allowing the pseudo action
          * to be runnable. */ 
         unordered_action->required_runnable_before = min_required_before;
-        update_action_flags(unordered_action, pe_action_requires_any, __FUNCTION__);
+        update_action_flags(unordered_action, pe_action_requires_any, __FUNCTION__, __LINE__);
 
         for (rIter = rsc_first->children; id && rIter; rIter = rIter->next) {
             resource_t *child = rIter->data;
@@ -647,7 +646,7 @@ tag_to_set(xmlNode * xml_obj, xmlNode ** rsc_set, const char * attr,
 }
 
 static gboolean unpack_rsc_location(xmlNode * xml_obj, resource_t * rsc_lh, const char * role,
-                             const char * score, pe_working_set_t * data_set, pe_re_match_data_t * match_data);
+                             const char * score, pe_working_set_t * data_set, pe_match_data_t * match_data);
 
 static gboolean
 unpack_simple_location(xmlNode * xml_obj, pe_working_set_t * data_set)
@@ -695,11 +694,16 @@ unpack_simple_location(xmlNode * xml_obj, pe_working_set_t * data_set)
             status = regexec(r_patt, r->id, nregs, pmatch, 0);
 
             if(invert == FALSE && status == 0) {
-                pe_re_match_data_t match_data = {
+                pe_re_match_data_t re_match_data = {
                                                 .string = r->id,
                                                 .nregs = nregs,
                                                 .pmatch = pmatch
                                                };
+                pe_match_data_t match_data = {
+                                                .re = &re_match_data,
+                                                .params = r->parameters,
+                                                .meta = r->meta,
+                                             };
                 crm_debug("'%s' matched '%s' for %s", r->id, value, id);
                 unpack_rsc_location(xml_obj, r, NULL, NULL, data_set, &match_data);
 
@@ -723,7 +727,7 @@ unpack_simple_location(xmlNode * xml_obj, pe_working_set_t * data_set)
 
 static gboolean
 unpack_rsc_location(xmlNode * xml_obj, resource_t * rsc_lh, const char * role,
-                    const char * score, pe_working_set_t * data_set, pe_re_match_data_t * match_data)
+                    const char * score, pe_working_set_t * data_set, pe_match_data_t * match_data)
 {
     gboolean empty = TRUE;
     rsc_to_node_t *location = NULL;
@@ -952,7 +956,7 @@ unpack_location(xmlNode * xml_obj, pe_working_set_t * data_set)
 }
 
 static int
-get_node_score(const char *rule, const char *score, gboolean raw, node_t * node)
+get_node_score(const char *rule, const char *score, gboolean raw, node_t * node, resource_t *rsc)
 {
     int score_f = 0;
 
@@ -963,7 +967,7 @@ get_node_score(const char *rule, const char *score, gboolean raw, node_t * node)
         score_f = char2score(score);
 
     } else {
-        const char *attr_score = g_hash_table_lookup(node->details->attrs, score);
+        const char *attr_score = pe_node_attribute_calculated(node, score, rsc);
 
         if (attr_score == NULL) {
             crm_debug("Rule %s: node %s did not have a value for %s",
@@ -981,7 +985,7 @@ get_node_score(const char *rule, const char *score, gboolean raw, node_t * node)
 
 static rsc_to_node_t *
 generate_location_rule(resource_t * rsc, xmlNode * rule_xml, const char *discovery, pe_working_set_t * data_set,
-                       pe_re_match_data_t * match_data)
+                       pe_match_data_t * match_data)
 {
     const char *rule_id = NULL;
     const char *score = NULL;
@@ -1027,9 +1031,9 @@ generate_location_rule(resource_t * rsc, xmlNode * rule_xml, const char *discove
         return NULL;
     }
 
-    if (match_data && match_data->nregs > 0 && match_data->pmatch[0].rm_so != -1) {
+    if (match_data && match_data->re && match_data->re->nregs > 0 && match_data->re->pmatch[0].rm_so != -1) {
         if (raw_score == FALSE) {
-            char *result = pe_expand_re_matches(score, match_data);
+            char *result = pe_expand_re_matches(score, match_data->re);
 
             if (result) {
                 score = (const char *) result;
@@ -1055,7 +1059,7 @@ generate_location_rule(resource_t * rsc, xmlNode * rule_xml, const char *discove
         for (gIter = match_L; gIter != NULL; gIter = gIter->next) {
             node_t *node = (node_t *) gIter->data;
 
-            node->weight = get_node_score(rule_id, score, raw_score, node);
+            node->weight = get_node_score(rule_id, score, raw_score, node, rsc);
         }
     }
 
@@ -1063,12 +1067,12 @@ generate_location_rule(resource_t * rsc, xmlNode * rule_xml, const char *discove
         int score_f = 0;
         node_t *node = (node_t *) gIter->data;
 
-        accept = pe_test_rule_re(rule_xml, node->details->attrs, RSC_ROLE_UNKNOWN, data_set->now, match_data);
+        accept = pe_test_rule_full(rule_xml, node->details->attrs, RSC_ROLE_UNKNOWN, data_set->now, match_data);
 
         crm_trace("Rule %s %s on %s", ID(rule_xml), accept ? "passed" : "failed",
                   node->details->uname);
 
-        score_f = get_node_score(rule_id, score, raw_score, node);
+        score_f = get_node_score(rule_id, score, raw_score, node, rsc);
 /* 			if(accept && score_f == -INFINITY) { */
 /* 				accept = FALSE; */
 /* 			} */
@@ -1263,7 +1267,7 @@ rsc_colocation_new(const char *id, const char *node_attr, int score,
     new_con->node_attribute = node_attr;
 
     if (node_attr == NULL) {
-        node_attr = "#" XML_ATTR_UNAME;
+        node_attr = CRM_ATTR_UNAME;
     }
 
     pe_rsc_trace(rsc_lh, "%s ==> %s (%s %d)", rsc_lh->id, rsc_rh->id, node_attr, score);
@@ -1788,7 +1792,7 @@ order_rsc_sets(const char *id, xmlNode * set1, xmlNode * set2, enum pe_order_kin
         action_t *unordered_action = get_pseudo_op(task, data_set);
 
         free(task);
-        update_action_flags(unordered_action, pe_action_requires_any, __FUNCTION__);
+        update_action_flags(unordered_action, pe_action_requires_any, __FUNCTION__, __LINE__);
 
         for (xml_rsc = __xml_first_child(set1); xml_rsc != NULL; xml_rsc = __xml_next_element(xml_rsc)) {
             if (!crm_str_eq((const char *)xml_rsc->name, XML_TAG_RESOURCE_REF, TRUE)) {
@@ -2204,8 +2208,7 @@ unpack_colocation_set(xmlNode * set, int score, pe_working_set_t * data_set)
     } else {
         /* Anti-colocating with every prior resource is
          * the only way to ensure the intuitive result
-         * (ie. that no-one in the set can run with anyone
-         * else in the set)
+         * (i.e. that no one in the set can run with anyone else in the set)
          */
 
         for (xml_rsc = __xml_first_child(set); xml_rsc != NULL; xml_rsc = __xml_next_element(xml_rsc)) {
@@ -2343,13 +2346,13 @@ unpack_simple_colocation(xmlNode * xml_obj, pe_working_set_t * data_set)
         crm_config_err("Invalid constraint '%s': No resource named '%s'", id, id_rh);
         return FALSE;
 
-    } else if (instance_lh && rsc_lh->variant < pe_clone) {
+    } else if (instance_lh && pe_rsc_is_clone(rsc_lh) == FALSE) {
         crm_config_err
             ("Invalid constraint '%s': Resource '%s' is not a clone but instance %s was requested",
              id, id_lh, instance_lh);
         return FALSE;
 
-    } else if (instance_rh && rsc_rh->variant < pe_clone) {
+    } else if (instance_rh && pe_rsc_is_clone(rsc_rh) == FALSE) {
         crm_config_err
             ("Invalid constraint '%s': Resource '%s' is not a clone but instance %s was requested",
              id, id_rh, instance_rh);
@@ -2585,10 +2588,19 @@ rsc_ticket_new(const char *id, resource_t * rsc_lh, ticket_t * ticket,
     new_rsc_ticket->role_lh = text2role(state_lh);
 
     if (safe_str_eq(loss_policy, "fence")) {
+        if (is_set(data_set->flags, pe_flag_stonith_enabled)) {
+            new_rsc_ticket->loss_policy = loss_ticket_fence;
+        } else {
+            crm_config_err("Resetting %s loss-policy to 'stop': fencing is not configured",
+                           ticket->id);
+            loss_policy = "stop";
+        }
+    }
+
+    if (new_rsc_ticket->loss_policy == loss_ticket_fence) {
         crm_debug("On loss of ticket '%s': Fence the nodes running %s (%s)",
                   new_rsc_ticket->ticket->id, new_rsc_ticket->rsc_lh->id,
                   role2text(new_rsc_ticket->role_lh));
-        new_rsc_ticket->loss_policy = loss_ticket_fence;
 
     } else if (safe_str_eq(loss_policy, "freeze")) {
         crm_debug("On loss of ticket '%s': Freeze %s (%s)",
@@ -2643,31 +2655,27 @@ unpack_rsc_ticket_set(xmlNode * set, ticket_t * ticket, const char *loss_policy,
 {
     xmlNode *xml_rsc = NULL;
     resource_t *resource = NULL;
-    const char *set_id = ID(set);
-    const char *role = crm_element_value(set, "role");
+    const char *set_id = NULL;
+    const char *role = NULL;
 
-    if (set == NULL) {
-        crm_config_err("No resource_set object to process.");
-        return FALSE;
-    }
+    CRM_CHECK(set != NULL, return FALSE);
+    CRM_CHECK(ticket != NULL, return FALSE);
 
+    set_id = ID(set);
     if (set_id == NULL) {
         crm_config_err("resource_set must have an id");
         return FALSE;
     }
 
-    if (ticket == NULL) {
-        crm_config_err("No dependented ticket specified for '%s'", set_id);
-        return FALSE;
-    }
+    role = crm_element_value(set, "role");
 
-    for (xml_rsc = __xml_first_child(set); xml_rsc != NULL; xml_rsc = __xml_next_element(xml_rsc)) {
-        if (crm_str_eq((const char *)xml_rsc->name, XML_TAG_RESOURCE_REF, TRUE)) {
-            EXPAND_CONSTRAINT_IDREF(set_id, resource, ID(xml_rsc));
-            pe_rsc_trace(resource, "Resource '%s' depends on ticket '%s'", resource->id,
-                         ticket->id);
-            rsc_ticket_new(set_id, resource, ticket, role, loss_policy, data_set);
-        }
+    for (xml_rsc = first_named_child(set, XML_TAG_RESOURCE_REF);
+         xml_rsc != NULL; xml_rsc = crm_next_same_xml(xml_rsc)) {
+
+        EXPAND_CONSTRAINT_IDREF(set_id, resource, ID(xml_rsc));
+        pe_rsc_trace(resource, "Resource '%s' depends on ticket '%s'",
+                     resource->id, ticket->id);
+        rsc_ticket_new(set_id, resource, ticket, role, loss_policy, data_set);
     }
 
     return TRUE;
@@ -2721,7 +2729,7 @@ unpack_simple_rsc_ticket(xmlNode * xml_obj, pe_working_set_t * data_set)
         crm_config_err("Invalid constraint '%s': No resource named '%s'", id, id_lh);
         return FALSE;
 
-    } else if (instance_lh && rsc_lh->variant < pe_clone) {
+    } else if (instance_lh && pe_rsc_is_clone(rsc_lh) == FALSE) {
         crm_config_err
             ("Invalid constraint '%s': Resource '%s' is not a clone but instance %s was requested",
              id, id_lh, instance_lh);
