@@ -75,18 +75,12 @@ struct rsc_ticket_s {
     int role_lh;
 };
 
-enum rsc_discover_e {
-    discover_always = 0,
-    discover_never,
-    discover_exclusive,
-};
-
 struct rsc_to_node_s {
     char *id;
     resource_t *rsc_lh;
 
     enum rsc_role_e role_filter;
-    enum rsc_discover_e discover_mode;
+    enum pe_discover_e discover_mode;
     GListPtr node_list_rh;      /* node_t* */
 };
 
@@ -145,6 +139,7 @@ extern int new_rsc_order(resource_t * lh_rsc, const char *lh_task,
     new_rsc_order(rsc1, CRMD_ACTION_STOP, rsc2, CRMD_ACTION_STOP, type, data_set)
 
 extern void graph_element_from_action(action_t * action, pe_working_set_t * data_set);
+extern void add_maintenance_update(pe_working_set_t *data_set);
 
 extern gboolean show_scores;
 extern int scores_log_level;
